# First steps

In this set of articles, you'll learn the core fundamentals of Dartness. To get familiar with the essential building
blocks
of Dartness applications, we'll build a basic CRUD application with features that cover a lot of ground at an
introductory
level.

## Prerequisites

Please make sure that [Dart SDK](https://dart.dev/get-dart) version >=2.17.0 is installed on your machine.

## Setup

```bash
$ dart create -t console your_project_name
```

1. Add dartness into the pubspec.yaml

```yaml
dependencies:
  dartness_server: ^0.4.0-alpha

dev_dependencies:
  build_runner: ^2.2.0
  dartness_generator: ^0.1.0-alpha
```

2. Create the file in "bin/main.dart" or whatever file that runs your application.

```dart
void main() async {
  final app = Dartness(
    port: 3000,
  );
  await app.create();
}
```

## Generate the code

Dartness uses the code generation feature provided by the package `code_builder`, this is the way that dart is able to
use reflection since `dart:mirrors` is unstable and not supported in order to compile your package.

In order to build the corresponding clases use the following command line:

```bash
$ dart run build_runner build
```

> HINT
>
> If you are coming from the JS or Java world this may sound odd, that's why I invite you to check the following
> articles:
> * [Code generation](https://infinum.com/handbook/flutter/basics/code-generation)
> * [dart:mirrors](https://api.dart.dev/stable/2.17.6/dart-mirrors/dart-mirrors-library.html)
> * [discontinuing support for dart:mirrors](https://github.com/dart-lang/sdk/issues/44489)

## Adding a controller

In order to create a controller, you need to import it and add it to the `Dartness` app. You can do
it in two different ways, either adding it when you create the dartness app by the `controllers` param or after
creating the app by `app.addController(controller)` method.

The param `controllers` accept the objects with a wrapper called `DartnessController`, this class receives 2 parameters,
the instance of the controller and the routes automatically generated by the class annotated by `@Controller`.

After that you execute `dart run build_runner build`, a new class will be generated that extends
from `DartnessController`, the name of this class will be the name of your class + 'DartnessController' appended
replacing 'Controller' to 'DartnessController', if the name of your class contains it, for example for `CityController`
a class called `CityDartnessController` will be generated. If your class doesn't contain 'Controller' in the name, '
DartnessController' will append to the name, for example for `AwesomeClass` a class
called `AwesomeClassDartnessController` will be created.

This new created class accepts an instance of the class from where have been created as param in the constructor, and it
must be used in the `controllers` param. You can see an example in the following code:

```dart
void main() async {
  final controllers = [
    CityDartnessController(CityController(CityService())),
  ];
  final app = Dartness(
    port: 3000,
    controllers: controllers,
  );
  await app.create();
}
```

## Running the application

Once the installation process is complete, you can run the following command at your OS command prompt to start the
application listening for inbound HTTP requests:

```bash
$ dart run bin/main.dart
Server listening on port 3000
```